#ifndef _GRIDEYE_H_
#define _GRIDEYE_H_

//GridEye.h driver adapted from Sparkfun library on github
//Copyright (C) Raymond Allen 2021
//MIT License

/*
  This is a library written for the Panasonic Grid-EYE AMG88
  SparkFun sells these at its website: www.sparkfun.com
  Do you like this library? Help support SparkFun. Buy a board!
  https://www.sparkfun.com/products/14568

  Written by Nick Poole @ SparkFun Electronics, January 11th, 2018

  The GridEYE from Panasonic is an 8 by 8 thermopile array capable
  of detecting temperature remotely at 64 discrete points.

  This library handles communication with the GridEYE and provides
  methods for manipulating temperature registers in Celsius,
  Fahrenheit and raw values.

  https://github.com/sparkfun/SparkFun_GridEYE_Arduino_Library
  */

i2c_Port* GridEye_i2cPort;
uint8_t GridEye_deviceAddress;

float GridEye_TempArray[64];
uint8_t GridEye_PaletteArray[64];
uint8_t GridEye_IntTemps[64];

uint8_t* GridEye_GetVgaMap()
{//read in all values and return in array of 8 bit palette color indexes
    //First get values and find min and max
    float mintemp = 0;
    float maxtemp = 1000;
    for (int i = 0; i < 64; i++)
    {
        GridEye_TempArray[i]= GridEye_getPixelTemperatureFahrenheit(i);
        if (GridEye_TempArray[i] > mintemp)
            mintemp = GridEye_TempArray[i];
        if (GridEye_TempArray[i] < maxtemp)
            maxtemp = GridEye_TempArray[i];
        GridEye_IntTemps[i] = GridEye_TempArray[i];
    }
    //convert to 8-bit indices
    for (int i = 0; i < 64; i++)
    {
        GridEye_PaletteArray[i] =(uint8_t) (255- (255.0 * (GridEye_TempArray[i] - mintemp)) / (maxtemp - mintemp));
    }
    return GridEye_PaletteArray;
}

float* GridEye_GetTemps()
{
	return GridEye_TempArray;
}

uint8_t* GridEye_GetTempMap()
{
	return GridEye_IntTemps;
}


//Registers
#define POWER_CONTROL_REGISTER        0x00
#define RESET_REGISTER                0x01
#define FRAMERATE_REGISTER            0x02
#define INT_CONTROL_REGISTER          0x03
#define STATUS_REGISTER               0x04
#define STATUS_CLEAR_REGISTER         0x05
#define AVERAGE_REGISTER              0x07
#define INT_LEVEL_REGISTER_UPPER_LSB  0x08
#define INT_LEVEL_REGISTER_UPPER_MSB  0x09
#define INT_LEVEL_REGISTER_LOWER_LSB  0x0A
#define INT_LEVEL_REGISTER_LOWER_MSB  0x0B
#define INT_LEVEL_REGISTER_HYST_LSB   0x0C
#define INT_LEVEL_REGISTER_HYST_MSB   0x0D
#define THERMISTOR_REGISTER_LSB       0x0E
#define THERMISTOR_REGISTER_MSB       0x0F
#define INT_TABLE_REGISTER_INT0       0x10
#define RESERVED_AVERAGE_REGISTER     0x1F
#define TEMPERATURE_REGISTER_START    0x80







//The default I2C address for the THING on the SparkX breakout is 0x69. 0x68 is also possible.
#define GridEye_DEFAULT_ADDRESS (0x69<<1)

	// Return values

	//By default use the default I2C addres, and use Wire port
	void GridEye_begin(i2c_Port* wirePort, uint8_t deviceAddress = GridEye_DEFAULT_ADDRESS);

	float GridEye_getPixelTemperature(unsigned char pixelAddr);
	int16_t GridEye_getPixelTemperatureRaw(unsigned char pixelAddr);
	float GridEye_getPixelTemperatureFahrenheit(unsigned char pixelAddr);

	float GridEye_getDeviceTemperature();
	int16_t GridEye_getDeviceTemperatureRaw();
	float GridEye_getDeviceTemperatureFahrenheit();

	void GridEye_setFramerate1FPS();
	void GridEye_setFramerate10FPS();
	bool GridEye_isFramerate10FPS();

	void GridEye_wake();
	void GridEye_sleep();
	void GridEye_standby60seconds();
	void GridEye_standby10seconds();

	void GridEye_interruptPinEnable();
	void GridEye_interruptPinDisable();
	void GridEye_setInterruptModeAbsolute();
	void GridEye_setInterruptModeDifference();
	bool GridEye_interruptPinEnabled();

	bool GridEye_interruptFlagSet();
	bool GridEye_pixelTemperatureOutputOK();
	bool GridEye_deviceTemperatureOutputOK();
	void GridEye_clearInterruptFlag();
	void GridEye_clearPixelTemperatureOverflow();
	void GridEye_clearDeviceTemperatureOverflow();
	void GridEye_clearAllOverflow();
	void GridEye_clearAllStatusFlags();

	bool GridEye_pixelInterruptSet(uint8_t pixelAddr);

	void GridEye_movingAverageEnable();
	void GridEye_movingAverageDisable();
	bool GridEye_movingAverageEnabled();

	void GridEye_setUpperInterruptValue(float DegreesC);
	void GridEye_setUpperInterruptValueRaw(int16_t regValue);
	void GridEye_setUpperInterruptValueFahrenheit(float DegreesF);

	void GridEye_setLowerInterruptValue(float DegreesC);
	void GridEye_setLowerInterruptValueRaw(int16_t regValue);
	void GridEye_setLowerInterruptValueFahrenheit(float DegreesF);

	void GridEye_setInterruptHysteresis(float DegreesC);
	void GridEye_setInterruptHysteresisRaw(int16_t regValue);
	void GridEye_setInterruptHysteresisFahrenheit(float DegreesF);

	float GridEye_getUpperInterruptValue();
	int16_t GridEye_getUpperInterruptValueRaw();
	float GridEye_getUpperInterruptValueFahrenheit();

	float GridEye_getLowerInterruptValue();
	int16_t GridEye_getLowerInterruptValueRaw();
	float GridEye_getLowerInterruptValueFahrenheit();

	float GridEye_getInterruptHysteresis();
	int16_t GridEye_getInterruptHysteresisRaw();
	float GridEye_getInterruptHysteresisFahrenheit();

	void GridEye_setRegister(unsigned char reg, unsigned char val);
	int16_t GridEye_getRegister(unsigned char reg, int8_t len);

	void GridEye_setI2CAddress(uint8_t addr); //Set the I2C address we read and write to


//Attempt communication with the device
//Return true if we got a 'Polo' back from Marco
void GridEye_begin(i2c_Port* wirePort, uint8_t deviceAddress)
{
    GridEye_deviceAddress = deviceAddress;
    GridEye_i2cPort = wirePort;
}

//Change the address we read and write to
void GridEye_setI2CAddress(uint8_t addr)
{
    _deviceAddress = addr;
}

/********************************************************
 * Functions for retreiving the temperature of
 * a single pixel.
 ********************************************************
 *
 * getPixelTemperature() - returns float Celsius
 *
 * getPixelTemperatureFahrenheit() - returns float Fahrenheit
 *
 * getPixelTemperatureRaw() - returns int16_t contents of
 *    both pixel temperature registers concatinated
 *
 ********************************************************/

float GridEye_getPixelTemperature(unsigned char pixelAddr)
{

    // Temperature registers are numbered 128-255
    // Each pixel has a lower and higher register
    unsigned char pixelLowRegister = TEMPERATURE_REGISTER_START + (2 * pixelAddr);
    int16_t temperature =GridEye_getRegister(pixelLowRegister, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        //temperature &= ~(1 << 11);
        
        //temperature = temperature * -1.0;
        temperature |= 0b1111 << 12;  //RJA fixed this
    }

    float DegreesC = temperature * 0.25;

    return DegreesC;

}

float GridEye_getPixelTemperatureFahrenheit(unsigned char pixelAddr)
{

    // Temperature registers are numbered 128-255
    // Each pixel has a lower and higher register
    unsigned char pixelLowRegister = TEMPERATURE_REGISTER_START + (2 * pixelAddr);
    int16_t temperature =GridEye_getRegister(pixelLowRegister, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        //temperature &= ~(1 << 11);
        //temperature = temperature * -1;
        temperature |= 0b1111 << 12;  //RJA fixed this
    }

    float DegreesF = (temperature * 0.25) * 1.8 + 32;

    return DegreesF;

}

uint16_t GridEye_getPixelTemperatureRaw(unsigned char pixelAddr)
{

    // Temperature registers are numbered 128-255
    // Each pixel has a lower and higher register
    unsigned char pixelLowRegister = TEMPERATURE_REGISTER_START + (2 * pixelAddr);
    uint16_t temperature =GridEye_getRegister(pixelLowRegister, 2);
    printf("result=%X\n", temperature);

    return temperature;

}

/********************************************************
 * Functions for retreiving the temperature of
 * the device according to the embedded thermistor.
 ********************************************************
 *
 * getDeviceTemperature() - returns float Celsius
 *
 * getDeviceTemperatureFahrenheit() - returns float Fahrenheit
 *
 * getDeviceTemperatureRaw() - returns int16_t contents of
 *    both thermistor temperature registers concatinated
 *
 ********************************************************/

float GridEye_getDeviceTemperature()
{

    int16_t temperature =GridEye_getRegister(THERMISTOR_REGISTER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float realTemperature = temperature * 0.0625;

    return realTemperature;

}

float GridEye_getDeviceTemperatureFahrenheit()
{

    int16_t temperature =GridEye_getRegister(THERMISTOR_REGISTER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float realTemperatureF = (temperature * 0.0625) * 1.8 + 32;

    return realTemperatureF;

}

int16_t GridEye_getDeviceTemperatureRaw()
{

    int16_t temperature =GridEye_getRegister(THERMISTOR_REGISTER_LSB, 2);

    return temperature;

}

/********************************************************
 * Functions for manipulating Framerate
 ********************************************************
 *
 * Internal framerate of the device is always 10fps
 * When operating in 1FPS mode, each frame is an average
 * of 10 readings.
 *
 * setFramerate1FPS() - sets framerate to 1 Frame per Second
 *
 * setFramerate10FPS() - sets framerate to 10 Frames per Second
 *
 * isFramerate10FPS() - returns true if framerate is currently
 *    set to 10 Frames per Second (device default)
 *
 ********************************************************/

void GridEye_setFramerate1FPS()
{
   GridEye_setRegister(FRAMERATE_REGISTER, 1);
}

void GridEye_setFramerate10FPS()
{
   GridEye_setRegister(FRAMERATE_REGISTER, 0);
}

bool GridEye_isFramerate10FPS()
{

    if (getRegister(FRAMERATE_REGISTER, 1) == 0) {
        return true;
    }
    else {
        return false;
    }

}

/********************************************************
 * Functions for manipulating Operating Mode
 ********************************************************
 *
 * Device defaults to normal mode on reset.
 * When the device is in standby mode, the temperature
 * register is only updated intermittently.
 *
 * wake() - returns device to normal mode from any
 *    other state.
 *
 * sleep() - puts device into sleep mode, temperature
 *    register is not updated
 *
 * standby60seconds() - puts device into standby mode
 *    with 60 second update frequency
 *
 * standby10seconds() - puts device into standby mode
 *    with 10 second update frequency
 *
 ********************************************************/

void GridEye_wake() {

   GridEye_setRegister(POWER_CONTROL_REGISTER, 0x00);

}

void GridEye_sleep() {

   GridEye_setRegister(POWER_CONTROL_REGISTER, 0x10);

}

void GridEye_standby60seconds() {

   GridEye_setRegister(POWER_CONTROL_REGISTER, 0x20);

}

void GridEye_standby10seconds() {

   GridEye_setRegister(POWER_CONTROL_REGISTER, 0x21);

}

/********************************************************
 * Functions for manipulating Interrupt Control Register
 ********************************************************
 *
 * interruptPinEnable() - Enable INT pin to pull low on
 *    interrupt flag
 *
 * interruptPinDisable() - Put INT pin into Hi-Z state
 *
 * setInterruptModeAbsolute() - Set interrupt mode to
 *    "Absolute Value" mode
 *
 * setInterruptModeDifference() - Set interrupt mode to
 *    "Difference" mode
 *
 * interruptPinEnabled() - returns true if the INT pin
 *    is enabled. Returns false if INT pin is in Hi-Z
 *
 ********************************************************/

void GridEye_interruptPinEnable() {

    int16_t ICRValue =GridEye_getRegister(INT_CONTROL_REGISTER, 1);

    ICRValue |= (1 << 0);

   GridEye_setRegister(INT_CONTROL_REGISTER, ICRValue & 0xFF);

}

void GridEye_interruptPinDisable() {

    int16_t ICRValue =GridEye_getRegister(INT_CONTROL_REGISTER, 1);

    ICRValue &= ~(1 << 0);

   GridEye_setRegister(INT_CONTROL_REGISTER, ICRValue & 0xFF);

}

void GridEye_setInterruptModeAbsolute() {

    int16_t ICRValue =GridEye_getRegister(INT_CONTROL_REGISTER, 1);

    ICRValue |= (1 << 1);

   GridEye_setRegister(INT_CONTROL_REGISTER, ICRValue & 0xFF);

}

void GridEye_setInterruptModeDifference() {

    int16_t ICRValue =GridEye_getRegister(INT_CONTROL_REGISTER, 1);

    ICRValue &= ~(1 << 1);

   GridEye_setRegister(INT_CONTROL_REGISTER, ICRValue & 0xFF);

}

bool GridEye_interruptPinEnabled() {

    int16_t ICRValue =GridEye_getRegister(INT_CONTROL_REGISTER, 1);
    if (ICRValue & (1 << 0)) {
        return true;
    }
    else {
        return false;
    }

}

/********************************************************
 * Functions for manipulating Status/Clear Registers
 ********************************************************
 *
 * interruptFlagSet() - returns true if there is an
 *    interrupt flag in the status register
 *
 * pixelTemperatureOutputOK() - returns false if temperature
 *    output overflow flag is present in status register
 *
 * deviceTemperatureOutputOK() - returns false if thermistor
 *    output overflow flag is present in status register
 *
 * clearInterruptFlag() - clears interrupt flag in the
 *    status register
 *
 * clearPixelTemperatureOverflow() - clears temperature
 *    output overflow flag in status register
 *
 * clearDeviceTemperatureOverflow() - clears thermistor
 *    output overflow flag in status register
 *
 * clearAllOverflow() - clears both thermistor and
 *    temperature overflow flags in status register but
 *    leaves interrupt flag untouched
 *
 * clearAllStatusFlags() - clears all flags in status
 *    register
 *
 ********************************************************/

bool GridEye_interruptFlagSet() {

    int16_t StatRegValue =GridEye_getRegister(STATUS_REGISTER, 1);
    if (StatRegValue & (1 << 1)) {
        return true;
    }
    else {
        return false;
    }

}

bool GridEye_pixelTemperatureOutputOK() {

    int16_t StatRegValue =GridEye_getRegister(STATUS_REGISTER, 1);
    if (StatRegValue & (1 << 2)) {
        return false;
    }
    else {
        return true;
    }

}

bool GridEye_deviceTemperatureOutputOK() {

    int16_t StatRegValue =GridEye_getRegister(STATUS_REGISTER, 1);
    if (StatRegValue & (1 << 3)) {
        return false;
    }
    else {
        return true;
    }

}

void GridEye_clearInterruptFlag() {

   GridEye_setRegister(STATUS_CLEAR_REGISTER, 0x02);

}

void GridEye_clearPixelTemperatureOverflow() {

   GridEye_setRegister(STATUS_CLEAR_REGISTER, 0x04);

}

void GridEye_clearDeviceTemperatureOverflow() {

   GridEye_setRegister(STATUS_CLEAR_REGISTER, 0x08);

}

void GridEye_clearAllOverflow() {

   GridEye_setRegister(STATUS_CLEAR_REGISTER, 0x0C);

}

void GridEye_clearAllStatusFlags() {

   GridEye_setRegister(STATUS_CLEAR_REGISTER, 0x0E);

}

/********************************************************
 * Function for reading Interrupt Table Register
 ********************************************************
 *
 * pixelInterruptSet() - Returns true if interrupt flag
 * is set for the specified pixel
 *
 ********************************************************/

bool GridEye_pixelInterruptSet(uint8_t pixelAddr) {

    unsigned char interruptTableRegister = INT_TABLE_REGISTER_INT0 + (pixelAddr / 8);
    uint8_t pixelPosition = (pixelAddr % 8);

    int16_t interruptTableRow =GridEye_getRegister(interruptTableRegister, 1);

    if (interruptTableRow & (1 << pixelPosition)) {
        return true;
    }
    else {
        return false;
    }

}

/********************************************************
 * Functions for manipulating Average Register
 ********************************************************
 *
 * Moving Average Mode enable and disable are only
 * referenced in some of the documentation for this
 * device but not in all documentation. Requires writing
 * in sequence to a reserved register. I'm not sure it
 * does anything.
 *
 * movingAverageEnable() - enable "Twice Moving Average"
 *
 * movingAverageDisable() - disable "Twice Moving Average"
 *
 * movingAverageEnabled() - returns true if enabled
 *
 ********************************************************/

void GridEye_movingAverageEnable() {

   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x50);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x45);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x57);
   GridEye_setRegister(AVERAGE_REGISTER, 0x20);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x00);

}

void GridEye_movingAverageDisable() {

   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x50);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x45);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x57);
   GridEye_setRegister(AVERAGE_REGISTER, 0x00);
   GridEye_setRegister(RESERVED_AVERAGE_REGISTER, 0x00);

}

bool GridEye_movingAverageEnabled() {

    int16_t AVGRegValue =GridEye_getRegister(AVERAGE_REGISTER, 1);
    if (AVGRegValue & (1 << 5)) {
        return true;
    }
    else {
        return false;
    }

}

/********************************************************
 * Functions for manipulating Interrupt Level Register
 ********************************************************
 *
 * setUpperInterruptValue() - accepts float Celsius
 *
 * setUpperInterruptValueRaw() - accepts int16_t register
 *    configuration
 *
 * setUpperInterruptValueFahrenheit() - accepts float
 *    Fahrenheit
 *
 * setLowerInterruptValue() - accepts float Celsius
 *
 * setLowerInterruptValueRaw() - accepts int16_t register
 *    configuration
 *
 * setLowerInterruptValueFahrenheit() - accepts float
 *    Fahrenheit
 *
 * setInterruptHysteresis() - accepts float Celsius
 *
 * setInterruptHysteresisRaw() - accepts int16_t register
 *    configuration
 *
 * setInterruptHysteresisFahrenheit() - accepts float
 *    Fahrenheit
 *
 * getUpperInterruptValue() - returns float Celsius
 *
 * getUpperInterruptValueRaw() - returns int16_t register
 *    contents
 *
 * getUpperInterruptValueFahrenheit() - returns float
 *    Fahrenheit
 *
 * getLowerInterruptValue() - returns float Celsius
 *
 * getLowerInterruptValueRaw() - returns int16_t register
 *    contents
 *
 * getLowerInterruptValueFahrenheit() - returns float
 *    Fahrenheit
 *
 * getInterruptHysteresis() - returns float Celsius
 *
 * getInterruptHysteresisRaw() - returns int16_t register
 *    contents
 *
 * getInterruptHysteresisFahrenheit() - returns float
 *    Fahrenheit
 *
 ********************************************************/

void GridEye_setUpperInterruptValue(float DegreesC) {

    bool isNegative = false;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_MSB, temperature >> 8);

}

void GridEye_setUpperInterruptValueRaw(int16_t regValue) {

   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_LSB, regValue & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_MSB, regValue >> 8);

}

void GridEye_setUpperInterruptValueFahrenheit(float DegreesF) {

    bool isNegative = false;

    float DegreesC = (DegreesF - 32) / 1.8;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_UPPER_MSB, temperature >> 8);

}

void GridEye_setLowerInterruptValue(float DegreesC) {

    bool isNegative = false;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_MSB, temperature >> 8);

}

void GridEye_setLowerInterruptValueRaw(int16_t regValue) {

   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_LSB, regValue & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_MSB, regValue >> 8);

}

void GridEye_setLowerInterruptValueFahrenheit(float DegreesF) {

    bool isNegative = false;

    float DegreesC = (DegreesF - 32) / 1.8;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_LOWER_MSB, temperature >> 8);

}

void GridEye_setInterruptHysteresis(float DegreesC) {

    bool isNegative = false;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_MSB, temperature >> 8);

}

void GridEye_setInterruptHysteresisRaw(int16_t regValue) {

   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_LSB, regValue & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_MSB, regValue >> 8);

}

void GridEye_setInterruptHysteresisFahrenheit(float DegreesF) {

    bool isNegative = false;

    float DegreesC = (DegreesF - 32) / 1.8;

    if (DegreesC < 0) {
        DegreesC = abs(DegreesC);
        isNegative = true;
    }

    int16_t temperature = 0;
    temperature = round(DegreesC * 4);

    if (isNegative) {
        temperature = 0 - temperature;
        temperature |= (1 << 11);
    }

   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_LSB, temperature & 0xFF);
   GridEye_setRegister(INT_LEVEL_REGISTER_HYST_MSB, temperature >> 8);

}

float GridEye_getUpperInterruptValue()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_UPPER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesC = temperature * 0.25;

    return DegreesC;

}

int16_t GridEye_getUpperInterruptValueRaw()
{

    returnGridEye_getRegister(INT_LEVEL_REGISTER_UPPER_LSB, 2);

}

float GridEye_getUpperInterruptValueFahrenheit()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_UPPER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesF = (temperature * 0.25) * 1.8 + 32;

    return DegreesF;

}

float GridEye_getLowerInterruptValue()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_LOWER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesC = temperature * 0.25;

    return DegreesC;

}

float GridEye_getLowerInterruptValueFahrenheit()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_LOWER_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesF = (temperature * 0.25) * 1.8 + 32;

    return DegreesF;

}

int16_t GridEye_getLowerInterruptValueRaw()
{

    returnGridEye_getRegister(INT_LEVEL_REGISTER_LOWER_LSB, 2);

}

float GridEye_getInterruptHysteresis()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_HYST_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesC = temperature * 0.25;

    return DegreesC;

}

float GridEye_getInterruptHysteresisFahrenheit()
{

    int16_t temperature =GridEye_getRegister(INT_LEVEL_REGISTER_HYST_LSB, 2);

    // temperature is reported as 12-bit twos complement
    // check if temperature is negative
    if (temperature & (1 << 11))
    {
        // if temperature is negative, mask out the sign byte and 
        // make the float negative
        temperature &= ~(1 << 11);
        temperature = temperature * -1;
    }

    float DegreesF = (temperature * 0.25) * 1.8 + 32;

    return DegreesF;

}

int16_t GridEye_getInterruptHysteresisRaw()
{

    returnGridEye_getRegister(INT_LEVEL_REGISTER_HYST_LSB, 2);

}

/********************************************************
 * Functions for setting and getting registers over I2C
 ********************************************************
 *
 *GridEye_setRegister() - set unsigned char value at unsigned char register
 *
 *GridEye_getRegister() - get up to INT16 value from unsigned char register
 *
 ********************************************************/

void GridEye_setRegister(unsigned char reg, unsigned char val)
{
    i2c_WriteRegisters(GridEye_i2cPort, GridEye_DEFAULT_ADDRESS, reg, 1, (uint8_t*)&val);

    //_i2cPort->beginTransmission(_deviceAddress);
    //_i2cPort->write(reg);
    //_i2cPort->write(val);
    //_i2cPort->endTransmission();

}

uint16_t GridEye_getRegister(unsigned char reg, int8_t len)
{

    uint16_t result;

    uint8_t high = 0;
    uint8_t low;

    i2c_ReadRegisters(GridEye_i2cPort, GridEye_DEFAULT_ADDRESS, reg, 1, (uint8_t*)&low);
    if (len>1)
        i2c_ReadRegisters(GridEye_i2cPort, GridEye_DEFAULT_ADDRESS, reg+1, 1, (uint8_t*)&high);

    result = (high << 8) + low;


    return result;

}

#endif /*_GRIDEYE_H_*/
/* Nothing beyond this*/



